/* Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730 */

#include "stdio.h"
#include "linpack.h"

#include "Abs.c"

#include "Hilgen.c"             /* Test System Generator    */
#include "BLAS.c"               /* Basic Linear Algebra     */
#include "SGEFA.c"              /* LINPACK Factor           */
#include "SGECO.c"              /* LINPACK Condition        */
#include "SGEDI.c"              /* LINPACK Determinant and  */
#include "OUT.c"                /* SICE Output Routine      */

main ()

     /* Program:                                            */
     /*                                                     */
     /*    LINPACK SGECO and SGEDI Test Driver.             */
     /*                                                     */
     /* Version:                 Date:                      */
     /*                                                     */
     /*    C                        02/22/85                */
     /*                                                     */
     /* Description:                                        */
     /*                                                     */
     /*    Uses LINPACK SGECO to compute RCond for matrix   */
     /*    A and LINPACK SGEDI to compute determinant and   */
     /*    inverse of matrix A where A is set up as a       */
     /*    Hilbert matrix of specified order using SYSGEN.  */
     /*    Uses SGEFA and SGEDI to invert inverse to com-   */
     /*    with original.                                   */
     /*                                                     */
     /* Author:                                             */
     /*                                                     */
     /*    Adam Fritz                                       */
     /*    133 Main Street                                  */
     /*    Afton, New York 13730                            */
     /*                                                     */

{
   float SDOT() ;
   int  i, j ;
   float AA[ldx][ldx] ;
   float P[ldx][ldx] ;
                                /* Initialize */
   lda = ldx ;
                                /* Announcement */
   printf("LINPACK SGECO and SDEDI Test Program.") ;
   printf("\n") ;
                                /* Get Order */
   n = 0 ;
   while ((n < 1) || (n > lda))
   {
      printf("Order: ") ;
      scanf("%d", &n) ;
   }
                                /* Get Print Code */
   printf("Print Code: ") ;
   scanf("%d", &PrintCode) ;
                                /* Generate Test System */
   SYSGEN(A, lda, n, b) ;
   if ( PrintCode > 0 )
   {
      printf("\n") ;
      printf("Original System (by column):\n") ;
      printf("\n") ;
      OUT(A, lda, n, n) ;
      OUT(b, lda, n, 1) ;
   }
                                /* Copy Original by Rows */
   for ( i = 0; i < n; i++ )
      SCOPY(n, &A[i][0], 1, &AA[i][0], 1) ;
                                /* Compute Condition */
   SGECO(A, lda, n, IPvt, &RCond, Work) ;
   if ( PrintCode > 0 )
   {
      printf("\n") ;
      printf("RCond: %7e",RCond) ;
      RCond += 1.0 ;
      RCond -= 1.0 ;
      printf(", RCond: %7e\n", RCond) ;
      printf("\n") ;
   }
                                /* Compute Determinant and Inverse */
   if ( RCond != 0.0 )
   {
      JobCode = 11 ;
      SGEDI(A, lda, n, IPvt, Det, Work, &JobCode) ;
      if ( PrintCode > 0 )
      {
         printf("\n") ;
         printf("Inverse (by column)\n:") ;
         printf("\n") ;
         OUT(A, lda, n, n) ;
         printf("Determinant: %7fe%-3d\n", Det[0], (int) Det[1]) ;
      }
                                /* Original Times Inverse */
      for ( i = 0; i < n; i++ )
         for ( j = 0; j < n; j++ )
            P[i][j] = SDOT(n, &AA[i][0], 1, &A[0][j], lda) ;
      if ( PrintCode > 0 )
      {
         printf("\n") ;
         printf("Original times Inverse (by column):\n") ;
         printf("\n") ;
         OUT(P, lda, n, n) ;
      }
                                /* Try to Restore Original */
      SGEFA(A, lda, n, IPvt, &InfoCode) ;
      if ( InfoCode == 0 )
      {
         SGEDI(A, lda, n, IPvt, Det, Work, &JobCode) ;
         if ( PrintCode > 0 )
         {
            printf("Inverse of Inverse (by column):\n") ;
            printf("\n") ;
            OUT(A, lda, n, n) ;
         }
      }
      else
         printf("Error: Inverse is Singular.\n") ;
   }
                                /* Zero Diagonal Element Produced */
   else
      printf("Error: Original is Singular.\n") ;
                                /* Done */
   printf("End of Test.\n") ;

}

/* Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730 */
