/* Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730 */

#define TEN 10.0
#define iTEN 10

SGEDI ( A, lda, n, IPvt, Det, Work, JobCode )
   float A[][ldx] ;
   int lda, n ;
   int IPvt[] ;
   float Det[] ;
   float Work[] ;
   int *JobCode ;

      /* Program:                                           */
      /*                                                    */
      /*    SGEDI - computes the determinant and inverse of */
      /*            a matrix using the factors computed by  */
      /*            SGECO or SGEFA.                         */
      /*                                                    */
      /* Version:                 Date:                     */
      /*                                                    */
      /*    C                        02/22/85               */
      /*    FORTRAN                  08/14/78               */
      /*                                                    */
      /* Description:                                       */
      /*                                                    */
      /*    On entry;                                       */
      /*                                                    */
      /*       A      - Output from SGECO or SGEFA.         */
      /*       lda    - Leading dimension of matrix A.      */
      /*       n      - Order of matrix A.                  */
      /*       IPvt   - Pivot index vector from SGECO or    */
      /*                SGEFA.                              */
      /*       Work   - Work vector.  Contents destroyed.   */
      /*       JobCode -                                    */
      /*            =11 compute determinant and inverse.    */
      /*            =01 compute inverse only.               */
      /*            =10 compute determinant only.           */
      /*                                                    */
      /*    On return;                                      */
      /*                                                    */
      /*       A      - Inverse of original matrix if       */
      /*                requested, otherwise unchanged.     */
      /*       Det    - Determinant of original matrix if   */
      /*                requested, otherwise ignored;       */
      /*                   determinant = Det[1]*10**Det[2]  */
      /*                with;                               */
      /*                   1.0 <= abs(Det[1]) < 10.0        */
      /*                or                                  */
      /*                   Det[1] = 0.0.                    */
      /*                                                    */
      /*    Error condition;                                */
      /*                                                    */
      /*       Division by zero will occur if the input     */
      /*       factor contains a zero on the diagonal and   */
      /*       the inverse is requested. It will not occur  */
      /*       if the subroutines are called correctly and  */
      /*       if SGECO has set RCond > 0.0 or SGEFA has    */
      /*       set InfoCode = 0.                            */
      /*                                                    */
      /* Subprograms:                                       */
      /*                                                    */
      /*    SAXPY    from BLAS                              */
      /*    SSCAL    from BLAS                              */
      /*    SSWAP    from BLAS                              */
      /*                                                    */
      /* Authors:                                           */
      /*                                                    */
      /*    Cleve Moler                                     */
      /*    University of New Mexico                        */
      /*    Argonne National Laboratories                   */
      /*                                                    */
      /*    C and Pascal translations;                      */
      /*                                                    */
      /*    Adam Fritz                                      */
      /*    133 Main Street                                 */
      /*    Afton, New York 13730                           */

{
   float Abs() ;
   float d ;
   float e ;
   float t ;
   int i, j, k, l ;
   int kb, kp1, nm1 ;
                                /* Compute Determinant */
   if ( *JobCode/iTEN != 0 )
   {
      d = 1.0 ;
      e = 0.0 ;
      i = 0 ;
      do {
         if ( IPvt[i] != i )
            d = -d ;
         d = A[i][i] * d ;
         if ( d != 0.0 )
         {
            while ( Abs(d) < 1.0 )
            {
               d *= TEN ;
               e -= 1.0 ;
            }
            while ( Abs(d) >= 10.0 )
            {
               d /= TEN ;
               e += 1.0 ;
            }
         }
      } while ((++i < n) && (d != 0.0)) ;
      Det[0] = d ;
      Det[1] = e ;
   }
                                /* Compute inverse(U) */
   if ( (*JobCode % iTEN) != 0 )
      {
      for ( k = 0; k < n; k++ )
      {
         A[k][k] = 1.0/A[k][k] ;
         t = -A[k][k] ;
         SSCAL(k, t, &A[0][k], lda) ;
         if ( k < n-1 )
         {
            kp1 = k + 1 ;
            for ( j = kp1; j < n; j++ )
            {
               t = A[k][j] ;
               A[k][j] = 0.0 ;
               SAXPY(k+1, t, &A[0][k], lda, &A[0][j], lda) ;
            }
         }
      }
                                /* Form inverse(U) * inverse(L) */
      if ( n > 1 )
      {
         nm1 = n - 1 ;
         for ( kb = 0; kb < nm1; kb++ )
         {
            k = n - kb - 2 ;
            kp1 = k + 1 ;
            for ( i = kp1; i < n; i++ )
            {
               Work[i] = A[i][k] ;
               A[i][k] = 0.0 ;
            }
            for ( j = kp1; j < n; j++ )
            {
               t = Work[j] ;
               SAXPY(n, t, &A[0][j], lda, &A[0][k], lda) ;
            }
            l = IPvt[k] ;
            if ( l != k )
               SSWAP(n, &A[0][k], lda, &A[0][l], lda) ;
         }
      }
   }
}

/* Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730 */
