/* Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730 */

SGESL ( A, lda, n, IPvt, b, JobCode )
   float A[][ldx] ;
   int lda, n ;
   int IPvt[] ;
   float b[] ;
   int *JobCode ;

     /* Program:                                            */
     /*                                                     */
     /*    SGESL - Solve General Real Linear System.        */
     /*                                                     */
     /* Version:                 Date:                      */
     /*                                                     */
     /*     C                       02/22/85                */
     /*     FORTRAN                 08/14/78                */
     /*                                                     */
     /* Description:                                        */
     /*                                                     */
     /*    Solves real single precision system;             */
     /*                                                     */
     /*       A * x = b  or  trans(A) * x = b               */
     /*                                                     */
     /*    using factors computed by SGECO or SGEFA.        */
     /*                                                     */
     /*    On entry;                                        */
     /*                                                     */
     /*       A      - Output from SGECO or SGEFA           */
     /*       lda    - Leading dimension of matrix A        */
     /*       n      - Order of matrix A.                   */
     /*       IPvt   - Pivot index vector from SGECO or     */
     /*                SGEFA.                               */
     /*       b      - Right hand side vector               */
     /*       JobCode -                                     */
     /*             =0 solve  A * x = b ,                   */
     /*             >0 solve  trans(A) * x = b  where       */
     /*                trans(A) is the transpose.           */
     /*                                                     */
     /*    On exit;                                         */
     /*                                                     */
     /*       b      - Solution vector  x .                 */
     /*                                                     */
     /*    Error Condition;                                 */
     /*                                                     */
     /*       Division by zero will occur if the input      */
     /*       factor contains a zero on the diagonal. This  */
     /*       indicates singularity but can be caused by    */
     /*       improper arguments or improper setting of lda.*/
     /*       It will occur if the procedures are called    */
     /*       correctly and SGECO has set RCOND greater     */
     /*       than 0.0 or SGEFA has set InfoCode            */
     /*       different from 0.                             */
     /*                                                     */
     /*    To compute;                                      */
     /*                                                     */
     /*       inverse(A) * c                                */
     /*                                                     */
     /*    where  c  is a matrix with  p  columns;          */
     /*                                                     */
     /*       SGECO(A, lda, n, IPvt, RCOND, z) ;            */
     /*       if RCOND ... ok                               */
     /*          for j = 1 to p do                          */
     /*             SGESL(A, lda, n, IPvt, c[1,j], false) ; */
     /*                                                     */
     /* Subprograms:                                        */
     /*                                                     */
     /*    SAXPY    from BLAS                               */
     /*    SDOT     from BLAS                               */
     /*                                                     */
     /* Authors:                                            */
     /*                                                     */
     /*    Cleve Moler                                      */
     /*    University of New Mexico                         */
     /*    Argonne National Laboratories                    */
     /*                                                     */
     /*    C and Pascal translations;                       */
     /*                                                     */
     /*    Adam Fritz                                       */
     /*    133 Main Street                                  */
     /*    Afton, New York 13730                            */

{
   float SDOT() ;
   int k, kb, l, nm1 ;
   float t ;

   nm1 = n - 1 ;
   if ( *JobCode == 0 )
   {
                                /* Solve  A * x = b  */
      if ( nm1 > 0 )
      {
         for ( k = 0; k < nm1; k++ )
         {
            l = IPvt[k] ;
            t = b[l] ;
            if ( l != k ) {
               b[l] = b[k] ;
               b[k] = t ;
            }
            SAXPY(n-k-1, t, &A[k+1][k], lda, &b[k+1], 1) ;
         }
      }
                                /* Solve  U * x = y  */
      for ( kb = 0; kb < n; kb++ )
      {
         k = n - kb - 1 ;
         b[k] = b[k]/A[k][k] ;
         t = -b[k] ;
         SAXPY(k, t, &A[0][k], lda, &b[0], 1) ;
      }
   }
                                /* Solve  trans(A) * x = b  */
   else
   {
                                /* Solve  trans(U) * y = b  */
      for ( k = 0; k < n; k++ )
      {
         t = SDOT(k, &A[0][k], lda, &b[0], 1) ;
         b[k] = (b[k] - t)/A[k][k] ;
      }
                                /* Solve trans(L) * x = y  */
      if ( nm1 > 0 )
      {
         for ( kb = 0; kb < nm1; kb++ )
         {
            k = n - kb - 2 ;
            b[k] = b[k] + SDOT(n-k-1, &A[k+1][k], lda, &b[k+1], 1) ;
            l = IPvt[k] ;
            if ( l != k )
            {
               t = b[l] ;
               b[l] = b[k] ;
               b[k] = t ;
            }
         }
      }
   }
}

/* Copyright (C) 1985 Adam Fritz, 133 Main St., Afton, NY 13730 */
